@@ -186,16 +186,16 @@ class Word2vecEmbedding(Layer):
     """
 
     def __init__(
-            self,
-            vocabulary_size,
-            embedding_size,
-            num_sampled=64,
-            activate_nce_loss=True,
-            nce_loss_args=None,
-            E_init=tl.initializers.random_uniform(minval=-1.0, maxval=1.0),
-            nce_W_init=tl.initializers.truncated_normal(stddev=0.03),
-            nce_b_init=tl.initializers.constant(value=0.0),
-            name=None,  #'word2vec',
+        self,
+        vocabulary_size,
+        embedding_size,
+        num_sampled=64,
+        activate_nce_loss=True,
+        nce_loss_args=None,
+        E_init=tl.initializers.random_uniform(minval=-1.0, maxval=1.0),
+        nce_W_init=tl.initializers.truncated_normal(stddev=0.03),
+        nce_b_init=tl.initializers.constant(value=0.0),
+        name=None,  #'word2vec',
     ):
 
         super(Word2vecEmbedding, self).__init__(name)
@@ -284,10 +284,8 @@ class Word2vecEmbedding(Layer):
             The nce_cost is returned only if the nce_loss is used.
         """
 
-        if isinstance(inputs, list):
-            outputs = tf.nn.embedding_lookup(params=self.embeddings, ids=inputs[0])
-        else:
-            outputs = tf.nn.embedding_lookup(params=self.embeddings, ids=inputs)
+        ids = inputs[0] if isinstance(inputs, list) else inputs
+        outputs = tf.nn.embedding_lookup(params=self.embeddings, ids=ids)
 
         if use_nce_loss is True and not self.activate_nce_loss:
             raise AttributeError(
@@ -352,11 +350,11 @@ class Embedding(Layer):
     """
 
     def __init__(
-            self,
-            vocabulary_size,
-            embedding_size,
-            E_init=tl.initializers.random_uniform(-0.1, 0.1),
-            name=None,  #'embedding',
+        self,
+        vocabulary_size,
+        embedding_size,
+        E_init=tl.initializers.random_uniform(-0.1, 0.1),
+        name=None,  #'embedding',
     ):
         super(Embedding, self).__init__(name)
         self.vocabulary_size = vocabulary_size
@@ -446,12 +444,12 @@ class AverageEmbedding(Layer):
     """
 
     def __init__(
-            self,
-            vocabulary_size,
-            embedding_size,
-            pad_value=0,
-            E_init=tl.initializers.random_uniform(-0.1, 0.1),
-            name=None,  # 'average_embedding',
+        self,
+        vocabulary_size,
+        embedding_size,
+        pad_value=0,
+        E_init=tl.initializers.random_uniform(-0.1, 0.1),
+        name=None,  # 'average_embedding',
     ):
 
         super(AverageEmbedding, self).__init__(name)
