@@ -48,12 +48,12 @@ class LocalResponseNorm(Layer):
     """
 
     def __init__(
-            self,
-            depth_radius=None,
-            bias=None,
-            alpha=None,
-            beta=None,
-            name=None,  #'lrn',
+        self,
+        depth_radius=None,
+        bias=None,
+        alpha=None,
+        beta=None,
+        name=None,  #'lrn',
     ):
         # super(LocalResponseNorm, self).__init__(prev_layer=prev_layer, name=name)
         super().__init__(name)
@@ -92,7 +92,7 @@ def _bias_scale(x, b, data_format):
     if data_format == 'NHWC':
         return x * b
     elif data_format == 'NCHW':
-        return x * _to_channel_first_bias(b)
+        return x * b
     else:
         raise ValueError('invalid data_format: %s' % data_format)
 
@@ -102,13 +102,34 @@ def _bias_add(x, b, data_format):
     if data_format == 'NHWC':
         return tf.add(x, b)
     elif data_format == 'NCHW':
-        return tf.add(x, _to_channel_first_bias(b))
+        return tf.add(x, b)
     else:
         raise ValueError('invalid data_format: %s' % data_format)
 
 
+def _compute_shape(tensors):
+    if isinstance(tensors, list):
+        shape_mem = [t.get_shape().as_list() for t in tensors]
+    else:
+        shape_mem = tensors.get_shape().as_list()
+    return shape_mem
+
+
 def batch_normalization(x, mean, variance, offset, scale, variance_epsilon, data_format, name=None):
     """Data Format aware version of tf.nn.batch_normalization."""
+    if data_format == 'channels_last':
+        mean = tf.reshape(mean, [1] * (len(x.shape) - 1) + [-1])
+        variance = tf.reshape(variance, [1] * (len(x.shape) - 1) + [-1])
+        offset = tf.reshape(offset, [1] * (len(x.shape) - 1) + [-1])
+        scale = tf.reshape(scale, [1] * (len(x.shape) - 1) + [-1])
+    elif data_format == 'channels_first':
+        mean = tf.reshape(mean, [1] + [-1] + [1] * (len(x.shape) - 2))
+        variance = tf.reshape(variance, [1] + [-1] + [1] * (len(x.shape) - 2))
+        offset = tf.reshape(offset, [1] + [-1] + [1] * (len(x.shape) - 2))
+        scale = tf.reshape(scale, [1] + [-1] + [1] * (len(x.shape) - 2))
+    else:
+        raise ValueError('invalid data_format: %s' % data_format)
+
     with ops.name_scope(name, 'batchnorm', [x, mean, variance, scale, offset]):
         inv = math_ops.rsqrt(variance + variance_epsilon)
         if scale is not None:
@@ -182,18 +203,18 @@ class BatchNorm(Layer):
     """
 
     def __init__(
-            self,
-            decay=0.9,
-            epsilon=0.00001,
-            act=None,
-            is_train=False,
-            beta_init=tl.initializers.zeros(),
-            gamma_init=tl.initializers.random_normal(mean=1.0, stddev=0.002),
-            moving_mean_init=tl.initializers.zeros(),
-            moving_var_init=tl.initializers.zeros(),
-            num_features=None,
-            data_format='channels_last',
-            name=None,
+        self,
+        decay=0.9,
+        epsilon=0.00001,
+        act=None,
+        is_train=False,
+        beta_init=tl.initializers.zeros(),
+        gamma_init=tl.initializers.random_normal(mean=1.0, stddev=0.002),
+        moving_mean_init=tl.initializers.zeros(),
+        moving_var_init=tl.initializers.zeros(),
+        num_features=None,
+        data_format='channels_last',
+        name=None,
     ):
         super(BatchNorm, self).__init__(name=name, act=act)
         self.decay = decay
@@ -205,13 +226,9 @@ class BatchNorm(Layer):
         self.moving_var_init = moving_var_init
         self.num_features = num_features
 
+        self.axes = None
+
         if num_features is not None:
-            if not isinstance(self, BatchNorm1d) and not isinstance(self, BatchNorm2d) and not isinstance(self,
-                                                                                                          BatchNorm3d):
-                raise ValueError(
-                    "Please use BatchNorm1d or BatchNorm2d or BatchNorm3d instead of BatchNorm "
-                    "if you want to specify 'num_features'."
-                )
             self.build(None)
             self._built = True
 
@@ -234,21 +251,24 @@ class BatchNorm(Layer):
 
     def _get_param_shape(self, inputs_shape):
         if self.data_format == 'channels_last':
-            axis = len(inputs_shape) - 1
+            axis = -1
         elif self.data_format == 'channels_first':
             axis = 1
         else:
             raise ValueError('data_format should be either %s or %s' % ('channels_last', 'channels_first'))
 
         channels = inputs_shape[axis]
-        params_shape = [1] * len(inputs_shape)
-        params_shape[axis] = channels
+        params_shape = [channels]
 
-        axes = [i for i in range(len(inputs_shape)) if i != axis]
-        return params_shape, axes
+        return params_shape
+
+    def _check_input_shape(self, inputs):
+        inputs_shape = _compute_shape(inputs)
+        if len(inputs_shape) <= 1:
+            raise ValueError('expected input at least 2D, but got {}D input'.format(inputs.ndim))
 
     def build(self, inputs_shape):
-        params_shape, self.axes = self._get_param_shape(inputs_shape)
+        params_shape = [self.num_features] if self.num_features is not None else self._get_param_shape(inputs_shape)
 
         self.beta, self.gamma = None, None
         if self.beta_init:
@@ -265,7 +285,13 @@ class BatchNorm(Layer):
         )
 
     def forward(self, inputs):
-        mean, var = tf.nn.moments(inputs, self.axes, keepdims=True)
+        self._check_input_shape(inputs)
+
+        self.channel_axis = len(inputs.shape) - 1 if self.data_format == 'channels_last' else 1
+        if self.axes is None:
+            self.axes = [i for i in range(len(inputs.shape)) if i != self.channel_axis]
+
+        mean, var = tf.nn.moments(inputs, self.axes, keepdims=False)
         if self.is_train:
             # update moving_mean and moving_var
             self.moving_mean = moving_averages.assign_moving_average(
@@ -283,8 +309,8 @@ class BatchNorm(Layer):
 
 
 class BatchNorm1d(BatchNorm):
-    """The :class:`BatchNorm1d` applies Batch Normalization over 3D input (a mini-batch of 1D
-    inputs with additional channel dimension), of shape (N, L, C) or (N, C, L).
+    """The :class:`BatchNorm1d` applies Batch Normalization over 2D/3D input (a mini-batch of 1D
+    inputs (optional) with additional channel dimension), of shape (N, C) or (N, L, C) or (N, C, L).
     See more details in :class:`BatchNorm`.
 
     Examples
@@ -300,23 +326,10 @@ class BatchNorm1d(BatchNorm):
 
     """
 
-    def _get_param_shape(self, inputs_shape):
-        if self.data_format == 'channels_last':
-            axis = 2
-        elif self.data_format == 'channels_first':
-            axis = 1
-        else:
-            raise ValueError('data_format should be either %s or %s' % ('channels_last', 'channels_first'))
-
-        if self.num_features is None:
-            channels = inputs_shape[axis]
-        else:
-            channels = self.num_features
-        params_shape = [1] * 3
-        params_shape[axis] = channels
-
-        axes = [i for i in range(3) if i != axis]
-        return params_shape, axes
+    def _check_input_shape(self, inputs):
+        inputs_shape = _compute_shape(inputs)
+        if len(inputs_shape) != 2 and len(inputs_shape) != 3:
+            raise ValueError('expected input to be 2D or 3D, but got {}D input'.format(inputs.ndim))
 
 
 class BatchNorm2d(BatchNorm):
@@ -337,23 +350,10 @@ class BatchNorm2d(BatchNorm):
 
     """
 
-    def _get_param_shape(self, inputs_shape):
-        if self.data_format == 'channels_last':
-            axis = 3
-        elif self.data_format == 'channels_first':
-            axis = 1
-        else:
-            raise ValueError('data_format should be either %s or %s' % ('channels_last', 'channels_first'))
-
-        if self.num_features is None:
-            channels = inputs_shape[axis]
-        else:
-            channels = self.num_features
-        params_shape = [1] * 4
-        params_shape[axis] = channels
-
-        axes = [i for i in range(4) if i != axis]
-        return params_shape, axes
+    def _check_input_shape(self, inputs):
+        inputs_shape = _compute_shape(inputs)
+        if len(inputs_shape) != 4:
+            raise ValueError('expected input to be 4D, but got {}D input'.format(inputs.ndim))
 
 
 class BatchNorm3d(BatchNorm):
@@ -374,23 +374,10 @@ class BatchNorm3d(BatchNorm):
 
     """
 
-    def _get_param_shape(self, inputs_shape):
-        if self.data_format == 'channels_last':
-            axis = 4
-        elif self.data_format == 'channels_first':
-            axis = 1
-        else:
-            raise ValueError('data_format should be either %s or %s' % ('channels_last', 'channels_first'))
-
-        if self.num_features is None:
-            channels = inputs_shape[axis]
-        else:
-            channels = self.num_features
-        params_shape = [1] * 5
-        params_shape[axis] = channels
-
-        axes = [i for i in range(5) if i != axis]
-        return params_shape, axes
+    def _check_input_shape(self, inputs):
+        inputs_shape = _compute_shape(inputs)
+        if len(inputs_shape) != 5:
+            raise ValueError('expected input to be 5D, but got {}D input'.format(inputs.ndim))
 
 
 class InstanceNorm(Layer):
@@ -436,9 +423,9 @@ class InstanceNorm(Layer):
     """
 
     def __init__(
-            self, act=None, epsilon=0.00001, beta_init=tl.initializers.zeros(),
-            gamma_init=tl.initializers.random_normal(mean=1.0, stddev=0.002), num_features=None,
-            data_format='channels_last', name=None
+        self, act=None, epsilon=0.00001, beta_init=tl.initializers.zeros(),
+        gamma_init=tl.initializers.random_normal(mean=1.0, stddev=0.002), num_features=None,
+        data_format='channels_last', name=None
     ):
         super(InstanceNorm, self).__init__(name=name, act=act)
         self.epsilon = epsilon
@@ -631,21 +618,21 @@ class LayerNorm(Layer):
     """
 
     def __init__(
-            self,  #prev_layer,
-            center=True,
-            scale=True,
-            act=None,
-            # reuse=None,
-            # variables_collections=None,
-            # outputs_collections=None,
-            # trainable=True,
-            epsilon=1e-12,
-            begin_norm_axis=1,
-            begin_params_axis=-1,
-            beta_init=tl.initializers.zeros(),
-            gamma_init=tl.initializers.ones(),
-            data_format='channels_last',
-            name=None,
+        self,  #prev_layer,
+        center=True,
+        scale=True,
+        act=None,
+        # reuse=None,
+        # variables_collections=None,
+        # outputs_collections=None,
+        # trainable=True,
+        epsilon=1e-12,
+        begin_norm_axis=1,
+        begin_params_axis=-1,
+        beta_init=tl.initializers.zeros(),
+        gamma_init=tl.initializers.ones(),
+        data_format='channels_last',
+        name=None,
     ):
 
         # super(LayerNorm, self).__init__(prev_layer=prev_layer, act=act, name=name)
@@ -830,17 +817,17 @@ class SwitchNorm(Layer):
     """
 
     def __init__(
-            self,
-            act=None,
-            epsilon=1e-5,
-            beta_init=tl.initializers.constant(0.0),
-            gamma_init=tl.initializers.constant(1.0),
-            moving_mean_init=tl.initializers.zeros(),
-            # beta_init=tf.compat.v1.initializers.constant(0.0),
-            # gamma_init=tf.compat.v1.initializers.constant(1.0),
-            # moving_mean_init=tf.compat.v1.initializers.zeros(),
-            data_format='channels_last',
-            name=None,  #'switchnorm',
+        self,
+        act=None,
+        epsilon=1e-5,
+        beta_init=tl.initializers.constant(0.0),
+        gamma_init=tl.initializers.constant(1.0),
+        moving_mean_init=tl.initializers.zeros(),
+        # beta_init=tf.compat.v1.initializers.constant(0.0),
+        # gamma_init=tf.compat.v1.initializers.constant(1.0),
+        # moving_mean_init=tf.compat.v1.initializers.zeros(),
+        data_format='channels_last',
+        name=None,  #'switchnorm',
     ):
         # super(SwitchNorm, self).__init__(prev_layer=prev_layer, act=act, name=name)
         super().__init__(name, act=act)
