@@ -59,12 +59,12 @@ class PoolLayer(Layer):
     """
 
     def __init__(
-            self,
-            filter_size=(1, 2, 2, 1),
-            strides=(1, 2, 2, 1),
-            padding='SAME',
-            pool=tf.nn.max_pool,
-            name=None  # 'pool_pro',
+        self,
+        filter_size=(1, 2, 2, 1),
+        strides=(1, 2, 2, 1),
+        padding='SAME',
+        pool=tf.nn.max_pool,
+        name=None  # 'pool_pro',
     ):
         super().__init__(name)
         self.filter_size = filter_size
@@ -122,13 +122,13 @@ class MaxPool1d(Layer):
     """
 
     def __init__(
-            self,
-            filter_size=3,
-            strides=2,
-            padding='SAME',
-            data_format='channels_last',
-            dilation_rate=1,
-            name=None  # 'maxpool1d'
+        self,
+        filter_size=3,
+        strides=2,
+        padding='SAME',
+        data_format='channels_last',
+        dilation_rate=1,
+        name=None  # 'maxpool1d'
     ):
         super().__init__(name)
         self.filter_size = self._filter_size = filter_size
@@ -207,13 +207,13 @@ class MeanPool1d(Layer):
     """
 
     def __init__(
-            self,
-            filter_size=3,
-            strides=2,
-            padding='SAME',
-            data_format='channels_last',
-            dilation_rate=1,
-            name=None  # 'meanpool1d'
+        self,
+        filter_size=3,
+        strides=2,
+        padding='SAME',
+        data_format='channels_last',
+        dilation_rate=1,
+        name=None  # 'meanpool1d'
     ):
         super().__init__(name)
         self.filter_size = self._filter_size = filter_size
@@ -293,12 +293,12 @@ class MaxPool2d(Layer):
     """
 
     def __init__(
-            self,
-            filter_size=(3, 3),
-            strides=(2, 2),
-            padding='SAME',
-            data_format='channels_last',
-            name=None  # 'maxpool2d'
+        self,
+        filter_size=(3, 3),
+        strides=(2, 2),
+        padding='SAME',
+        data_format='channels_last',
+        name=None  # 'maxpool2d'
     ):
         super().__init__(name)
         self.filter_size = filter_size
@@ -324,17 +324,19 @@ class MaxPool2d(Layer):
         return s.format(classname=self.__class__.__name__, **self.__dict__)
 
     def build(self, inputs_shape=None):
-        self._strides = [1, self.strides[0], self.strides[1], 1]
         if self.data_format == 'channels_last':
+            self._strides = [1, self.strides[0], self.strides[1], 1]
             self.data_format = 'NHWC'
         elif self.data_format == 'channels_first':
             self.data_format = 'NCHW'
+            self._strides = [1, 1, self.strides[0], self.strides[1]]
         else:
             raise Exception("unsupported data format")
 
     def forward(self, inputs):
         outputs = tf.nn.max_pool(
-            input=inputs, ksize=self.filter_size, strides=self._strides, padding=self.padding, name=self.name
+            input=inputs, ksize=self.filter_size, strides=self._strides, padding=self.padding, name=self.name,
+            data_format=self.data_format
         )
         return outputs
 
@@ -366,12 +368,12 @@ class MeanPool2d(Layer):
     """
 
     def __init__(
-            self,
-            filter_size=(3, 3),
-            strides=(2, 2),
-            padding='SAME',
-            data_format='channels_last',
-            name=None  # 'meanpool2d'
+        self,
+        filter_size=(3, 3),
+        strides=(2, 2),
+        padding='SAME',
+        data_format='channels_last',
+        name=None  # 'meanpool2d'
     ):
         super().__init__(name)
         self.filter_size = filter_size
@@ -397,17 +399,19 @@ class MeanPool2d(Layer):
         return s.format(classname=self.__class__.__name__, **self.__dict__)
 
     def build(self, inputs_shape=None):
-        self._strides = [1, self.strides[0], self.strides[1], 1]
         if self.data_format == 'channels_last':
             self.data_format = 'NHWC'
+            self._strides = [1, self.strides[0], self.strides[1], 1]
         elif self.data_format == 'channels_first':
             self.data_format = 'NCHW'
+            self._strides = [1, 1, self.strides[0], self.strides[1]]
         else:
             raise Exception("unsupported data format")
 
     def forward(self, inputs):
         outputs = tf.nn.avg_pool(
-            input=inputs, ksize=self.filter_size, strides=self._strides, padding=self.padding, name=self.name
+            input=inputs, ksize=self.filter_size, strides=self._strides, padding=self.padding, name=self.name,
+            data_format=self.data_format
         )
         return outputs
 
@@ -444,12 +448,12 @@ class MaxPool3d(Layer):
     """
 
     def __init__(
-            self,
-            filter_size=(3, 3, 3),
-            strides=(2, 2, 2),
-            padding='VALID',
-            data_format='channels_last',
-            name=None  # 'maxpool3d'
+        self,
+        filter_size=(3, 3, 3),
+        strides=(2, 2, 2),
+        padding='VALID',
+        data_format='channels_last',
+        name=None  # 'maxpool3d'
     ):
         super().__init__(name)
         self.filter_size = filter_size
@@ -473,11 +477,12 @@ class MaxPool3d(Layer):
         return s.format(classname=self.__class__.__name__, **self.__dict__)
 
     def build(self, inputs_shape=None):
-        self._strides = [1, self.strides[0], self.strides[1], self.strides[2], 1]
         if self.data_format == 'channels_last':
             self.data_format = 'NDHWC'
+            self._strides = [1, self.strides[0], self.strides[1], self.strides[2], 1]
         elif self.data_format == 'channels_first':
             self.data_format = 'NCDHW'
+            self._strides = [1, 1, self.strides[0], self.strides[1], self.strides[2]]
         else:
             raise Exception("unsupported data format")
 
@@ -525,12 +530,12 @@ class MeanPool3d(Layer):
     """
 
     def __init__(
-            self,
-            filter_size=(3, 3, 3),
-            strides=(2, 2, 2),
-            padding='VALID',
-            data_format='channels_last',
-            name=None  # 'meanpool3d'
+        self,
+        filter_size=(3, 3, 3),
+        strides=(2, 2, 2),
+        padding='VALID',
+        data_format='channels_last',
+        name=None  # 'meanpool3d'
     ):
         super().__init__(name)
         self.filter_size = filter_size
@@ -554,11 +559,12 @@ class MeanPool3d(Layer):
         return s.format(classname=self.__class__.__name__, **self.__dict__)
 
     def build(self, inputs_shape=None):
-        self._strides = [1, self.strides[0], self.strides[1], self.strides[2], 1]
         if self.data_format == 'channels_last':
             self.data_format = 'NDHWC'
+            self._strides = [1, self.strides[0], self.strides[1], self.strides[2], 1]
         elif self.data_format == 'channels_first':
             self.data_format = 'NCDHW'
+            self._strides = [1, 1, self.strides[0], self.strides[1], self.strides[2]]
         else:
             raise Exception("unsupported data format")
 
@@ -595,9 +601,9 @@ class GlobalMaxPool1d(Layer):
     """
 
     def __init__(
-            self,
-            data_format="channels_last",
-            name=None  # 'globalmaxpool1d'
+        self,
+        data_format="channels_last",
+        name=None  # 'globalmaxpool1d'
     ):
         super().__init__(name)
 
@@ -651,9 +657,9 @@ class GlobalMeanPool1d(Layer):
     """
 
     def __init__(
-            self,
-            data_format='channels_last',
-            name=None  # 'globalmeanpool1d'
+        self,
+        data_format='channels_last',
+        name=None  # 'globalmeanpool1d'
     ):
         super().__init__(name)
         self.data_format = data_format
@@ -706,9 +712,9 @@ class GlobalMaxPool2d(Layer):
     """
 
     def __init__(
-            self,
-            data_format='channels_last',
-            name=None  # 'globalmaxpool2d'
+        self,
+        data_format='channels_last',
+        name=None  # 'globalmaxpool2d'
     ):
         super().__init__(name)
         self.data_format = data_format
@@ -761,9 +767,9 @@ class GlobalMeanPool2d(Layer):
     """
 
     def __init__(
-            self,
-            data_format='channels_last',
-            name=None  # 'globalmeanpool2d'
+        self,
+        data_format='channels_last',
+        name=None  # 'globalmeanpool2d'
     ):
         super().__init__(name)
 
@@ -817,9 +823,9 @@ class GlobalMaxPool3d(Layer):
     """
 
     def __init__(
-            self,
-            data_format='channels_last',
-            name=None  # 'globalmaxpool3d'
+        self,
+        data_format='channels_last',
+        name=None  # 'globalmaxpool3d'
     ):
         super().__init__(name)
 
@@ -873,9 +879,9 @@ class GlobalMeanPool3d(Layer):
     """
 
     def __init__(
-            self,
-            data_format='channels_last',
-            name=None  # 'globalmeanpool3d'
+        self,
+        data_format='channels_last',
+        name=None  # 'globalmeanpool3d'
     ):
         super().__init__(name)
         self.data_format = data_format
@@ -929,9 +935,9 @@ class CornerPool2d(Layer):
     """
 
     def __init__(
-            self,
-            mode='TopLeft',
-            name=None  # 'cornerpool2d'
+        self,
+        mode='TopLeft',
+        name=None  # 'cornerpool2d'
     ):
         super().__init__(name)
         self.mode = mode
