@@ -1,6 +1,7 @@
 #! /usr/bin/python
 # -*- coding: utf-8 -*-
 
+import numpy as np
 import tensorflow as tf
 
 import tensorlayer as tl
@@ -8,17 +9,12 @@ from tensorlayer import logging
 from tensorlayer.decorators import deprecated_alias
 from tensorlayer.layers.core import Layer
 
-# from tensorflow.python.ops import array_ops
-# from tensorflow.python.util.tf_inspect import getfullargspec
-# from tensorflow.contrib.rnn import stack_bidirectional_dynamic_rnn
-# from tensorflow.python.ops.rnn_cell import LSTMStateTuple
-
-# from tensorlayer.layers.core import LayersConfig
-# from tensorlayer.layers.core import TF_GRAPHKEYS_VARIABLES
-
 # TODO: uncomment
 __all__ = [
     'RNN',
+    'SimpleRNN',
+    'GRURNN',
+    'LSTMRNN',
     'BiRNN',
     # 'ConvRNNCell',
     # 'BasicConvLSTMCell',
@@ -26,8 +22,7 @@ __all__ = [
     'retrieve_seq_length_op',
     'retrieve_seq_length_op2',
     'retrieve_seq_length_op3',
-    # 'target_mask_op',
-    # 'Seq2Seq',
+    'target_mask_op',
 ]
 
 
@@ -42,24 +37,31 @@ class RNN(Layer):
         A RNN cell implemented by tf.keras
             - E.g. tf.keras.layers.SimpleRNNCell, tf.keras.layers.LSTMCell, tf.keras.layers.GRUCell
             - Note TF2.0+, TF1.0+ and TF1.0- are different
+
     return_last_output : boolean
         Whether return last output or all outputs in a sequence.
+
             - If True, return the last output, "Sequence input and single output"
             - If False, return all outputs, "Synced sequence input and output"
             - In other word, if you want to stack more RNNs on this layer, set to False
+
         In a dynamic model, `return_last_output` can be updated when it is called in customised forward().
         By default, `False`.
     return_seq_2d : boolean
         Only consider this argument when `return_last_output` is `False`
+
             - If True, return 2D Tensor [batch_size * n_steps, n_hidden], for stacking Dense layer after it.
             - If False, return 3D Tensor [batch_size, n_steps, n_hidden], for stacking multiple RNN after it.
+
         In a dynamic model, `return_seq_2d` can be updated when it is called in customised forward().
         By default, `False`.
     return_last_state: boolean
         Whether to return the last state of the RNN cell. The state is a list of Tensor.
         For simple RNN and GRU, last_state = [last_output]; For LSTM, last_state = [last_output, last_cell_state]
+
             - If True, the layer will return outputs and the final state of the cell.
             - If False, the layer will return outputs only.
+
         In a dynamic model, `return_last_state` can be updated when it is called in customised forward().
         By default, `False`.
     in_channels: int
@@ -74,6 +76,7 @@ class RNN(Layer):
     For synced sequence input and output, see `PTB example <https://github.com/tensorlayer/tensorlayer/blob/master/example/tutorial_ptb_lstm.py>`__
 
     A simple regression model below.
+    
     >>> inputs = tl.layers.Input([batch_size, num_steps, embedding_size])
     >>> rnn_out, lstm_state = tl.layers.RNN(
     >>>     cell=tf.keras.layers.LSTMCell(units=hidden_size, dropout=0.1),
@@ -85,6 +88,7 @@ class RNN(Layer):
     >>> # If LSTMCell is applied, the rnn_state is [h, c] where h the hidden state and c the cell state of LSTM.
 
     A stacked RNN model.
+    
     >>> inputs = tl.layers.Input([batch_size, num_steps, embedding_size])
     >>> rnn_out1 = tl.layers.RNN(
     >>>     cell=tf.keras.layers.SimpleRNNCell(units=hidden_size, dropout=0.1),
@@ -97,21 +101,50 @@ class RNN(Layer):
     >>> outputs = tl.layers.Dense(n_units=1)(rnn_out2)
     >>> rnn_model = tl.models.Model(inputs=inputs, outputs=outputs)
 
+    An example if the sequences have different length and contain padding.
+    Similar to the DynamicRNN in TL 1.x.
+
+    If the `sequence_length` is provided in RNN's forwarding and both `return_last_output` and `return_last_state`
+    are set as `True`, the forward function will automatically ignore the paddings. Note that if `return_last_output`
+    is set as `False`, the synced sequence outputs will still include outputs which correspond with paddings,
+    but users are free to select which slice of outputs to be used in following procedure.
+
+    The `sequence_length` should be a list of integers which indicates the length of each sequence.
+    It is recommended to
+    `tl.layers.retrieve_seq_length_op3 <https://tensorlayer.readthedocs.io/en/latest/modules/layers.html#compute-sequence-length-3>`__
+    to calculate the `sequence_length`.
+
+    >>> data = [[[1], [2], [0], [0], [0]], [[1], [2], [3], [0], [0]], [[1], [2], [6], [1], [1]]]
+    >>> data = tf.convert_to_tensor(data, dtype=tf.float32)
+    >>> class DynamicRNNExample(tl.models.Model):
+    >>>     def __init__(self):
+    >>>         super(DynamicRNNExample, self).__init__()
+    >>>         self.rnnlayer = tl.layers.RNN(
+    >>>             cell=tf.keras.layers.SimpleRNNCell(units=6, dropout=0.1), in_channels=1, return_last_output=True,
+    >>>             return_last_state=True
+    >>>         )
+    >>>     def forward(self, x):
+    >>>         z, s = self.rnnlayer(x, sequence_length=tl.layers.retrieve_seq_length_op3(x))
+    >>>         return z, s
+    >>> model = DynamicRNNExample()
+    >>> model.eval()
+    >>> output, state = model(data)
+
+
     Notes
     -----
     Input dimension should be rank 3 : [batch_size, n_steps, n_features], if no, please see layer :class:`Reshape`.
 
-
     """
 
     def __init__(
-            self,
-            cell,
-            return_last_output=False,
-            return_seq_2d=False,
-            return_last_state=True,
-            in_channels=None,
-            name=None,  # 'rnn'
+        self,
+        cell,
+        return_last_output=False,
+        return_seq_2d=False,
+        return_last_state=True,
+        in_channels=None,
+        name=None,  # 'rnn'
     ):
 
         super(RNN, self).__init__(name=name)
@@ -156,49 +189,118 @@ class RNN(Layer):
             self._trainable_weights.append(var)
 
     # @tf.function
-    def forward(self, inputs, initial_state=None, **kwargs):
+    def forward(self, inputs, sequence_length=None, initial_state=None, **kwargs):
         """
         Parameters
         ----------
         inputs : input tensor
             The input of a network
+        sequence_length: None or list of integers
+            The actual length of each sequence in batch without padding.
+            If provided, when `return_last_output` and `return_last_state` are `True`,
+            the RNN will perform in the manner of a dynamic RNN, i.e.
+            the RNN will return the actual last output / state without padding.
         initial_state : None or list of Tensor (RNN State)
             If None, `initial_state` is zero state.
+
         **kwargs: dict
             Some attributes can be updated during forwarding
             such as `return_last_output`, `return_seq_2d`, `return_last_state`.
         """
-
         if kwargs:
             for attr in kwargs:
                 if attr in self.__dict__:
                     setattr(self, attr, kwargs[attr])
 
-        if self.return_last_output:
+        batch_size = inputs.get_shape().as_list()[0]
+        total_steps = inputs.get_shape().as_list()[1]
+
+        # checking the type and values of sequence_length
+        if sequence_length is not None:
+            if isinstance(sequence_length, list):
+                pass
+            elif isinstance(sequence_length, tf.Tensor):
+                pass
+            elif isinstance(sequence_length, np.ndarray):
+                sequence_length = sequence_length.tolist()
+            else:
+                raise TypeError(
+                    "The argument sequence_length should be either None or a list of integers. "
+                    "Type got %s" % type(sequence_length)
+                )
+            if (len(sequence_length) != batch_size):
+                raise ValueError(
+                    "The argument sequence_length should contain %d " % batch_size +
+                    "elements indicating the initial length of each sequence, but got only %d. " % len(sequence_length)
+                )
+            for i in sequence_length:
+                if not (type(i) is int or (isinstance(i, tf.Tensor) and i.dtype.is_integer)):
+                    raise TypeError(
+                        "The argument sequence_length should be either None or a list of integers. "
+                        "One element of sequence_length has the type %s" % type(i)
+                    )
+                if i > total_steps:
+                    raise ValueError(
+                        "The actual length of a sequence should not be longer than "
+                        "that of the longest sequence (total steps) in this mini-batch. "
+                        "Total steps of this mini-batch %d, " % total_steps +
+                        "but got an actual length of a sequence %d" % i
+                    )
+
+        sequence_length = tl.layers.retrieve_seq_length_op3(inputs)
+
+        sequence_length = [i - 1 if i >= 1 else 0 for i in sequence_length]
+
+        # set warning
+        # if (not self.return_last_output) and sequence_length is not None:
+        #     warnings.warn(
+        #         'return_last_output is set as %s ' % self.return_last_output +
+        #         'When sequence_length is provided, it is recommended to set as True. ' +
+        #         'Otherwise, padding will be considered while RNN is forwarding.'
+        #     )
+
+        # return the last output, iterating each seq including padding ones. No need to store output during each
+        # time step.
+        if self.return_last_output and sequence_length is None:
             outputs = [-1]
         else:
             outputs = list()
 
+        # initialize the states if provided
         states = initial_state if initial_state is not None else self.cell.get_initial_state(inputs)
         if not isinstance(states, list):
             states = [states]
 
-        total_steps = inputs.get_shape().as_list()[1]
+        stored_states = list()
 
+        # initialize the cell
         self.cell.reset_dropout_mask()
         self.cell.reset_recurrent_dropout_mask()
 
+        # recurrent computation
+        # FIXME: if sequence_length is provided (dynamic rnn), only iterate max(sequence_length) times.
         for time_step in range(total_steps):
 
             cell_output, states = self.cell.call(inputs[:, time_step, :], states, training=self.is_train)
+            stored_states.append(states)
 
-            if self.return_last_output:
+            if self.return_last_output and sequence_length is None:
                 outputs[-1] = cell_output
             else:
                 outputs.append(cell_output)
 
-        if self.return_last_output:
+        # prepare to return results
+        if self.return_last_output and sequence_length is None:
             outputs = outputs[-1]
+
+        elif self.return_last_output and sequence_length is not None:
+            outputs = tf.convert_to_tensor(outputs)
+            outputs = tf.gather(outputs, sequence_length, axis=0)
+
+            outputs_without_padding = []
+            for i in range(batch_size):
+                outputs_without_padding.append(outputs[i][i][:])
+            outputs = tf.convert_to_tensor(outputs_without_padding)
         else:
             if self.return_seq_2d:
                 # PTB tutorial: stack dense layer after that, or compute the cost from the output
@@ -209,13 +311,254 @@ class RNN(Layer):
                 # 3D Tensor [batch_size, n_steps, n_hidden]
                 outputs = tf.reshape(tf.concat(outputs, 1), [-1, total_steps, self.cell.units])
 
-        if self.return_last_state:
+        if self.return_last_state and sequence_length is None:
+            return outputs, states
+        elif self.return_last_state and sequence_length is not None:
+
+            stored_states = tf.convert_to_tensor(stored_states)
+            stored_states = tf.gather(stored_states, sequence_length, axis=0)
+
+            states = []
+            for i in range(stored_states.shape[1]):
+                states.append(tf.convert_to_tensor([stored_states[b, i, b, :] for b in range(batch_size)]))
+
             return outputs, states
         else:
             return outputs
 
 
-# TODO: write tl.layers.SimpleRNN, tl.layers.GRU, tl.layers.LSTM
+class SimpleRNN(RNN):
+    """
+    The :class:`SimpleRNN` class is a fixed length recurrent layer for implementing simple RNN.
+
+    Parameters
+    ----------
+    units: int
+        Positive integer, the dimension of hidden space.
+    return_last_output : boolean
+        Whether return last output or all outputs in a sequence.
+            - If True, return the last output, "Sequence input and single output"
+            - If False, return all outputs, "Synced sequence input and output"
+            - In other word, if you want to stack more RNNs on this layer, set to False
+
+        In a dynamic model, `return_last_output` can be updated when it is called in customised forward().
+        By default, `False`.
+    return_seq_2d : boolean
+        Only consider this argument when `return_last_output` is `False`
+            - If True, return 2D Tensor [batch_size * n_steps, n_hidden], for stacking Dense layer after it.
+            - If False, return 3D Tensor [batch_size, n_steps, n_hidden], for stacking multiple RNN after it.
+
+        In a dynamic model, `return_seq_2d` can be updated when it is called in customised forward().
+        By default, `False`.
+    return_last_state: boolean
+        Whether to return the last state of the RNN cell. The state is a list of Tensor.
+        For simple RNN, last_state = [last_output]
+
+            - If True, the layer will return outputs and the final state of the cell.
+            - If False, the layer will return outputs only.
+
+        In a dynamic model, `return_last_state` can be updated when it is called in customised forward().
+        By default, `False`.
+    in_channels: int
+        Optional, the number of channels of the previous layer which is normally the size of embedding.
+        If given, the layer will be built when init.
+        If None, it will be automatically detected when the layer is forwarded for the first time.
+    name : str
+        A unique layer name.
+    `**kwargs`:
+        Advanced arguments to configure the simple RNN cell.
+        Please check tf.keras.layers.SimpleRNNCell.
+
+    Examples
+    --------
+
+    A simple regression model below.
+
+    >>> inputs = tl.layers.Input([batch_size, num_steps, embedding_size])
+    >>> rnn_out, lstm_state = tl.layers.SimpleRNN(
+    >>>     units=hidden_size, dropout=0.1, # both units and dropout are used to configure the simple rnn cell.
+    >>>     in_channels=embedding_size,
+    >>>     return_last_output=True, return_last_state=True, name='simplernn'
+    >>> )(inputs)
+    >>> outputs = tl.layers.Dense(n_units=1)(rnn_out)
+    >>> rnn_model = tl.models.Model(inputs=inputs, outputs=[outputs, rnn_state[0]], name='rnn_model')
+
+    Notes
+    -----
+    Input dimension should be rank 3 : [batch_size, n_steps, n_features], if no, please see layer :class:`Reshape`.
+
+    """
+
+    def __init__(
+        self,
+        units,
+        return_last_output=False,
+        return_seq_2d=False,
+        return_last_state=True,
+        in_channels=None,
+        name=None,  # 'simplernn'
+        **kwargs
+    ):
+        super(SimpleRNN, self).__init__(
+            cell=tf.keras.layers.SimpleRNNCell(units=units, **kwargs), return_last_output=return_last_output,
+            return_seq_2d=return_seq_2d, return_last_state=return_last_state, in_channels=in_channels, name=name
+        )
+
+
+class GRURNN(RNN):
+    """
+    The :class:`GRURNN` class is a fixed length recurrent layer for implementing RNN with GRU cell.
+
+    Parameters
+    ----------
+    units: int
+        Positive integer, the dimension of hidden space.
+    return_last_output : boolean
+        Whether return last output or all outputs in a sequence.
+            - If True, return the last output, "Sequence input and single output"
+            - If False, return all outputs, "Synced sequence input and output"
+            - In other word, if you want to stack more RNNs on this layer, set to False
+
+        In a dynamic model, `return_last_output` can be updated when it is called in customised forward().
+        By default, `False`.
+    return_seq_2d : boolean
+        Only consider this argument when `return_last_output` is `False`
+            - If True, return 2D Tensor [batch_size * n_steps, n_hidden], for stacking Dense layer after it.
+            - If False, return 3D Tensor [batch_size, n_steps, n_hidden], for stacking multiple RNN after it.
+
+        In a dynamic model, `return_seq_2d` can be updated when it is called in customised forward().
+        By default, `False`.
+    return_last_state: boolean
+        Whether to return the last state of the RNN cell. The state is a list of Tensor.
+        For GRU, last_state = [last_output]
+
+            - If True, the layer will return outputs and the final state of the cell.
+            - If False, the layer will return outputs only.
+
+        In a dynamic model, `return_last_state` can be updated when it is called in customised forward().
+        By default, `False`.
+    in_channels: int
+        Optional, the number of channels of the previous layer which is normally the size of embedding.
+        If given, the layer will be built when init.
+        If None, it will be automatically detected when the layer is forwarded for the first time.
+    name : str
+        A unique layer name.
+    `**kwargs`:
+        Advanced arguments to configure the GRU cell.
+        Please check tf.keras.layers.GRUCell.
+
+    Examples
+    --------
+
+    A simple regression model below.
+
+    >>> inputs = tl.layers.Input([batch_size, num_steps, embedding_size])
+    >>> rnn_out, lstm_state = tl.layers.GRURNN(
+    >>>     units=hidden_size, dropout=0.1, # both units and dropout are used to configure the GRU cell.
+    >>>     in_channels=embedding_size,
+    >>>     return_last_output=True, return_last_state=True, name='grurnn'
+    >>> )(inputs)
+    >>> outputs = tl.layers.Dense(n_units=1)(rnn_out)
+    >>> rnn_model = tl.models.Model(inputs=inputs, outputs=[outputs, rnn_state[0]], name='rnn_model')
+
+    Notes
+    -----
+    Input dimension should be rank 3 : [batch_size, n_steps, n_features], if no, please see layer :class:`Reshape`.
+
+    """
+
+    def __init__(
+        self,
+        units,
+        return_last_output=False,
+        return_seq_2d=False,
+        return_last_state=True,
+        in_channels=None,
+        name=None,  # 'grurnn'
+        **kwargs
+    ):
+        super(GRURNN, self).__init__(
+            cell=tf.keras.layers.GRUCell(units=units, **kwargs), return_last_output=return_last_output,
+            return_seq_2d=return_seq_2d, return_last_state=return_last_state, in_channels=in_channels, name=name
+        )
+
+
+class LSTMRNN(RNN):
+    """
+    The :class:`LSTMRNN` class is a fixed length recurrent layer for implementing RNN with LSTM cell.
+
+    Parameters
+    ----------
+    units: int
+        Positive integer, the dimension of hidden space.
+    return_last_output : boolean
+        Whether return last output or all outputs in a sequence.
+            - If True, return the last output, "Sequence input and single output"
+            - If False, return all outputs, "Synced sequence input and output"
+            - In other word, if you want to stack more RNNs on this layer, set to False
+
+        In a dynamic model, `return_last_output` can be updated when it is called in customised forward().
+        By default, `False`.
+    return_seq_2d : boolean
+        Only consider this argument when `return_last_output` is `False`
+            - If True, return 2D Tensor [batch_size * n_steps, n_hidden], for stacking Dense layer after it.
+            - If False, return 3D Tensor [batch_size, n_steps, n_hidden], for stacking multiple RNN after it.
+
+        In a dynamic model, `return_seq_2d` can be updated when it is called in customised forward().
+        By default, `False`.
+    return_last_state: boolean
+        Whether to return the last state of the RNN cell. The state is a list of Tensor.
+        For LSTM, last_state = [last_output, last_cell_state]
+
+            - If True, the layer will return outputs and the final state of the cell.
+            - If False, the layer will return outputs only.
+
+        In a dynamic model, `return_last_state` can be updated when it is called in customised forward().
+        By default, `False`.
+    in_channels: int
+        Optional, the number of channels of the previous layer which is normally the size of embedding.
+        If given, the layer will be built when init.
+        If None, it will be automatically detected when the layer is forwarded for the first time.
+    name : str
+        A unique layer name.
+    `**kwargs`:
+        Advanced arguments to configure the LSTM cell.
+        Please check tf.keras.layers.LSTMCell.
+
+    Examples
+    --------
+
+    A simple regression model below.
+
+    >>> inputs = tl.layers.Input([batch_size, num_steps, embedding_size])
+    >>> rnn_out, lstm_state = tl.layers.LSTMRNN(
+    >>>     units=hidden_size, dropout=0.1, # both units and dropout are used to configure the LSTM cell.
+    >>>     in_channels=embedding_size,
+    >>>     return_last_output=True, return_last_state=True, name='grurnn'
+    >>> )(inputs)
+    >>> outputs = tl.layers.Dense(n_units=1)(rnn_out)
+    >>> rnn_model = tl.models.Model(inputs=inputs, outputs=[outputs, rnn_state[0]], name='rnn_model')
+
+    Notes
+    -----
+    Input dimension should be rank 3 : [batch_size, n_steps, n_features], if no, please see layer :class:`Reshape`.
+
+    """
+
+    def __init__(
+        self,
+        units,
+        return_last_output=False,
+        return_seq_2d=False,
+        return_last_state=True,
+        in_channels=None,
+        name=None,  # 'lstmrnn'
+        **kwargs
+    ):
+        super(LSTMRNN, self).__init__(
+            cell=tf.keras.layers.LSTMCell(units=units, **kwargs), return_last_output=return_last_output,
+            return_seq_2d=return_seq_2d, return_last_state=return_last_state, in_channels=in_channels, name=name
+        )
 
 
 class BiRNN(Layer):
@@ -237,6 +580,7 @@ class BiRNN(Layer):
         Whether to return the last state of the two cells. The state is a list of Tensor.
             - If True, the layer will return outputs, the final state of `fw_cell` and the final state of `bw_cell`.
             - If False, the layer will return outputs only.
+
         In a dynamic model, `return_last_state` can be updated when it is called in customised forward().
         By default, `False`.
     in_channels: int
@@ -249,6 +593,7 @@ class BiRNN(Layer):
     Examples
     --------
     A simple regression model below.
+    
     >>> inputs = tl.layers.Input([batch_size, num_steps, embedding_size])
     >>> # the fw_cell and bw_cell can be different
     >>> rnnlayer = tl.layers.BiRNN(
@@ -266,6 +611,7 @@ class BiRNN(Layer):
     >>> rnn_model = tl.models.Model(inputs=inputs, outputs=[outputs, rnn_out, rnn_fw_state[0], rnn_bw_state[0]])
 
     A stacked BiRNN model.
+    
     >>> inputs = tl.layers.Input([batch_size, num_steps, embedding_size])
     >>> rnn_out1 = tl.layers.BiRNN(
     >>>     fw_cell=tf.keras.layers.SimpleRNNCell(units=hidden_size, dropout=0.1),
@@ -281,7 +627,6 @@ class BiRNN(Layer):
     >>> outputs = tl.layers.Reshape([-1, num_steps])(dense)
     >>> rnn_model = tl.models.Model(inputs=inputs, outputs=outputs)
 
-
     Notes
     -----
     Input dimension should be rank 3 : [batch_size, n_steps, n_features]. If not, please see layer :class:`Reshape`.
@@ -289,13 +634,13 @@ class BiRNN(Layer):
     """
 
     def __init__(
-            self,
-            fw_cell,
-            bw_cell,
-            return_seq_2d=False,
-            return_last_state=False,
-            in_channels=None,
-            name=None,  # 'birnn'
+        self,
+        fw_cell,
+        bw_cell,
+        return_seq_2d=False,
+        return_last_state=False,
+        in_channels=None,
+        name=None,  # 'birnn'
     ):
         super(BiRNN, self).__init__(name)
 
@@ -418,6 +763,7 @@ class BiRNN(Layer):
             return outputs
 
 
+'''
 class ConvRNNCell(object):
     """Abstract object representing an Convolutional RNN Cell."""
 
@@ -612,10 +958,12 @@ class ConvLSTM(Layer):
             - If True, return the last output, "Sequence input and single output".
             - If False, return all outputs, "Synced sequence input and output".
             - In other word, if you want to stack more RNNs on this layer, set to False.
+
     return_seq_2d : boolean
         Only consider this argument when `return_last_output` is `False`
             - If True, return 2D Tensor [n_example, n_hidden], for stacking DenseLayer after it.
             - If False, return 3D Tensor [n_example/n_steps, n_steps, n_hidden], for stacking multiple RNN after it.
+
     name : str
         A unique layer name.
 
@@ -729,6 +1077,8 @@ class ConvLSTM(Layer):
         self._add_layers(self.outputs)
         self._add_params(rnn_variables)
 
+'''
+
 
 # @tf.function
 def retrieve_seq_length_op(data):
@@ -789,7 +1139,7 @@ def retrieve_seq_length_op2(data):
     >>>         [1,2,6,1,0]]
     >>> data = tf.convert_to_tensor(data, dtype=tf.float32)
     >>> length = tl.layers.retrieve_seq_length_op2(data)
-    [2 3 4]
+    tensor([2 3 4])
 
     """
     return tf.reduce_sum(input_tensor=tf.cast(tf.greater(data, tf.zeros_like(data)), tf.int32), axis=1)
@@ -817,25 +1167,25 @@ def retrieve_seq_length_op3(data, pad_val=0):
     >>>         [[1],[2],[6],[1],[0]]]
     >>> data = tf.convert_to_tensor(data, dtype=tf.float32)
     >>> length = tl.layers.retrieve_seq_length_op3(data)
-    [2, 3, 4]
+    tensor([2, 3, 4])
     >>> data = [[[1,2],[2,2],[1,2],[1,2],[0,0]],
     >>>         [[2,3],[2,4],[3,2],[0,0],[0,0]],
     >>>         [[3,3],[2,2],[5,3],[1,2],[0,0]]]
     >>> data = tf.convert_to_tensor(data, dtype=tf.float32)
     >>> length = tl.layers.retrieve_seq_length_op3(data)
-    [4, 3, 4]
+    tensor([4, 3, 4])
     >>> data = [[1,2,0,0,0],
     >>>         [1,2,3,0,0],
     >>>         [1,2,6,1,0]]
     >>> data = tf.convert_to_tensor(data, dtype=tf.float32)
     >>> length = tl.layers.retrieve_seq_length_op3(data)
-    [2, 3, 4]
+    tensor([2, 3, 4])
     >>> data = [['hello','world','','',''],
     >>>         ['hello','world','tensorlayer','',''],
     >>>         ['hello','world','tensorlayer','2.0','']]
     >>> data = tf.convert_to_tensor(data, dtype=tf.string)
     >>> length = tl.layers.retrieve_seq_length_op3(data, pad_val='')
-    [2, 3, 4]
+    tensor([2, 3, 4])
 
     """
     data_shape_size = data.get_shape().ndims
@@ -854,200 +1204,64 @@ def retrieve_seq_length_op3(data, pad_val=0):
         )
 
 
-def target_mask_op(data, pad_val=0):  # HangSheng: return tensor for mask,if input is tf.string
-    """Return tensor for mask, if input is ``tf.string``."""
+def target_mask_op(data, pad_val=0):
+    """ Return the mask of the input sequence data based on the padding values.
+
+    Parameters
+    -----------
+    data : tf.Tensor
+        A tensor with 2 or 3 dimensions.
+    pad_val: int, float, string, etc
+        The value that represent padding. By default, 0. For tf.string, you may use empty string.
+
+    Examples
+    -----------
+    >>> data = [['hello', 'world', '', '', ''],
+    >>>         ['hello', 'world', 'tensorlayer', '', ''],
+    >>>         ['hello', 'world', 'tensorlayer', '2.0', '']]
+    >>> data = tf.convert_to_tensor(data, dtype=tf.string)
+    >>> mask = tl.layers.target_mask_op(data, pad_val='')
+    >>> print(mask)
+    tf.Tensor(
+    [[1 1 0 0 0]
+     [1 1 1 0 0]
+     [1 1 1 1 0]], shape=(3, 5), dtype=int32)
+    >>> data = [[[1], [0], [0], [0], [0]],
+    >>>         [[1], [2], [3], [0], [0]],
+    >>>         [[1], [2], [0], [1], [0]]]
+    >>> data = tf.convert_to_tensor(data, dtype=tf.float32)
+    >>> mask = tl.layers.target_mask_op(data)
+    >>> print(mask)
+    tf.Tensor(
+    [[1 0 0 0 0]
+     [1 1 1 0 0]
+     [1 1 0 1 0]], shape=(3, 5), dtype=int32)
+    >>> data = [[[0,0],[2,2],[1,2],[1,2],[0,0]],
+    >>>         [[2,3],[2,4],[3,2],[1,0],[0,0]],
+    >>>         [[3,3],[0,1],[5,3],[1,2],[0,0]]]
+    >>> data = tf.convert_to_tensor(data, dtype=tf.float32)
+    >>> mask = tl.layers.target_mask_op(data)
+    >>> print(mask)
+    tf.Tensor(
+    [[0 1 1 1 0]
+     [1 1 1 1 0]
+     [1 1 1 1 0]], shape=(3, 5), dtype=int32)
+    """
+
+    if not isinstance(data, tf.Tensor):
+        raise AttributeError("target_mask_op: the type of input data should be tf.Tensor but got %s." % type(data))
     data_shape_size = data.get_shape().ndims
     if data_shape_size == 3:
         return tf.cast(tf.reduce_any(input_tensor=tf.not_equal(data, pad_val), axis=2), dtype=tf.int32)
     elif data_shape_size == 2:
         return tf.cast(tf.not_equal(data, pad_val), dtype=tf.int32)
     elif data_shape_size == 1:
-        raise ValueError("target_mask_op: data has wrong shape!")
+        raise ValueError(
+            "target_mask_op: data_shape %s is not supported. "
+            "The shape of data should have 2 or 3 dims." % (data.get_shape())
+        )
     else:
-        raise ValueError("target_mask_op: handling data_shape_size %s hasn't been implemented!" % (data_shape_size))
-
-
-class Seq2Seq(Layer):
-    """
-    The :class:`Seq2Seq` class is a simple :class:`DynamicRNNLayer` based Seq2seq layer without using `tl.contrib.seq2seq <https://www.tensorflow.org/api_guides/python/contrib.seq2seq>`__.
-    See `Model <https://camo.githubusercontent.com/9e88497fcdec5a9c716e0de5bc4b6d1793c6e23f/687474703a2f2f73757269796164656570616e2e6769746875622e696f2f696d672f736571327365712f73657132736571322e706e67>`__
-    and `Sequence to Sequence Learning with Neural Networks <https://arxiv.org/abs/1409.3215>`__.
-
-    - Please check this example `Chatbot in 200 lines of code <https://github.com/tensorlayer/seq2seq-chatbot>`__.
-    - The Author recommends users to read the source code of :class:`DynamicRNNLayer` and :class:`Seq2Seq`.
-
-    Parameters
-    ----------
-    net_encode_in : :class:`Layer`
-        Encode sequences, [batch_size, None, n_features].
-    net_decode_in : :class:`Layer`
-        Decode sequences, [batch_size, None, n_features].
-    cell_fn : TensorFlow cell function
-        A TensorFlow core RNN cell
-            - see `RNN Cells in TensorFlow <https://www.tensorflow.org/api_docs/python/>`__
-            - Note TF1.0+ and TF1.0- are different
-    cell_init_args : dictionary or None
-        The arguments for the cell initializer.
-    n_hidden : int
-        The number of hidden units in the layer.
-    initializer : initializer
-        The initializer for the parameters.
-    encode_sequence_length : tensor
-        For encoder sequence length, see :class:`DynamicRNNLayer` .
-    decode_sequence_length : tensor
-        For decoder sequence length, see :class:`DynamicRNNLayer` .
-    initial_state_encode : None or RNN state
-        If None, `initial_state_encode` is zero state, it can be set by placeholder or other RNN.
-    initial_state_decode : None or RNN state
-        If None, `initial_state_decode` is the final state of the RNN encoder, it can be set by placeholder or other RNN.
-    dropout : tuple of float or int
-        The input and output keep probability (input_keep_prob, output_keep_prob).
-            - If one int, input and output keep probability are the same.
-    n_layer : int
-        The number of RNN layers, default is 1.
-    return_seq_2d : boolean
-        Only consider this argument when `return_last_output` is `False`
-            - If True, return 2D Tensor [n_example, 2 * n_hidden], for stacking DenseLayer after it.
-            - If False, return 3D Tensor [n_example/n_steps, n_steps, 2 * n_hidden], for stacking multiple RNN after it.
-    name : str
-        A unique layer name.
-
-    Attributes
-    ------------
-    outputs : tensor
-        The output of RNN decoder.
-    initial_state_encode : tensor or StateTuple
-        Initial state of RNN encoder.
-    initial_state_decode : tensor or StateTuple
-        Initial state of RNN decoder.
-    final_state_encode : tensor or StateTuple
-        Final state of RNN encoder.
-    final_state_decode : tensor or StateTuple
-        Final state of RNN decoder.
-
-    Notes
-    --------
-    - How to feed data: `Sequence to Sequence Learning with Neural Networks <https://arxiv.org/pdf/1409.3215v3.pdf>`__
-    - input_seqs : ``['how', 'are', 'you', '<PAD_ID>']``
-    - decode_seqs : ``['<START_ID>', 'I', 'am', 'fine', '<PAD_ID>']``
-    - target_seqs : ``['I', 'am', 'fine', '<END_ID>', '<PAD_ID>']``
-    - target_mask : ``[1, 1, 1, 1, 0]``
-    - related functions : tl.prepro <pad_sequences, precess_sequences, sequences_add_start_id, sequences_get_mask>
-
-    Examples
-    ----------
-    >>> from tensorlayer.layers import *
-    >>> batch_size = 32
-    >>> encode_seqs = tf.placeholder(dtype=tf.int64, shape=[batch_size, None], name="encode_seqs")
-    >>> decode_seqs = tf.placeholder(dtype=tf.int64, shape=[batch_size, None], name="decode_seqs")
-    >>> target_seqs = tf.placeholder(dtype=tf.int64, shape=[batch_size, None], name="target_seqs")
-    >>> target_mask = tf.placeholder(dtype=tf.int64, shape=[batch_size, None], name="target_mask") # tl.prepro.sequences_get_mask()
-    >>> with tf.variable_scope("model"):
-    >>>     # for chatbot, you can use the same embedding layer,
-    >>>     # for translation, you may want to use 2 seperated embedding layers
-    >>>     with tf.variable_scope("embedding") as vs:
-    >>>         net_encode = EmbeddingInput(
-    ...                 inputs = encode_seqs,
-    ...                 vocabulary_size = 10000,
-    ...                 embedding_size = 200,
-    ...                 name = 'seq_embedding')
-    >>>         vs.reuse_variables()
-    >>>         net_decode = EmbeddingInput(
-    ...                 inputs = decode_seqs,
-    ...                 vocabulary_size = 10000,
-    ...                 embedding_size = 200,
-    ...                 name = 'seq_embedding')
-    >>>     net = Seq2Seq(net_encode, net_decode,
-    ...             cell_fn = tf.contrib.rnn.BasicLSTMCell,
-    ...             n_hidden = 200,
-    ...             initializer = tf.random_uniform_initializer(-0.1, 0.1),
-    ...             encode_sequence_length = retrieve_seq_length_op2(encode_seqs),
-    ...             decode_sequence_length = retrieve_seq_length_op2(decode_seqs),
-    ...             initial_state_encode = None,
-    ...             dropout = None,
-    ...             n_layer = 1,
-    ...             return_seq_2d = True,
-    ...             name = 'seq2seq')
-    >>> net_out = Dense(net, n_units=10000, act=None, name='output')
-    >>> e_loss = tl.cost.cross_entropy_seq_with_mask(logits=net_out.outputs, target_seqs=target_seqs, input_mask=target_mask, return_details=False, name='cost')
-    >>> y = tf.nn.softmax(net_out.outputs)
-    >>> net_out.print_params(False)
-
-    """
-
-    def __init__(
-            self,
-            net_encode_in,
-            net_decode_in,
-            cell_fn,  #tf.nn.rnn_cell.LSTMCell,
-            cell_init_args=None,
-            n_hidden=256,
-            initializer=tf.compat.v1.initializers.random_uniform(-0.1, 0.1),
-            encode_sequence_length=None,
-            decode_sequence_length=None,
-            initial_state_encode=None,
-            initial_state_decode=None,
-            dropout=None,
-            n_layer=1,
-            return_seq_2d=False,
-            name='seq2seq',
-    ):
-        super(Seq2Seq,
-              self).__init__(prev_layer=[net_encode_in, net_decode_in], cell_init_args=cell_init_args, name=name)
-
-        if self.cell_init_args:
-            self.cell_init_args['state_is_tuple'] = True  # 'use_peepholes': True,
-
-        if cell_fn is None:
-            raise ValueError("cell_fn cannot be set to None")
-
-        if 'GRU' in cell_fn.__name__:
-            try:
-                cell_init_args.pop('state_is_tuple')
-            except Exception:
-                logging.warning("pop state_is_tuple fails.")
-
-        logging.info(
-            "[*] Seq2Seq %s: n_hidden: %d cell_fn: %s dropout: %s n_layer: %d" %
-            (self.name, n_hidden, cell_fn.__name__, dropout, n_layer)
+        raise ValueError(
+            "target_mask_op: handling data_shape %s hasn't been implemented! "
+            "The shape of data should have 2 or 3 dims" % (data.get_shape())
         )
-
-        with tf.compat.v1.variable_scope(name):
-            # tl.layers.set_name_reuse(reuse)
-            # network = InputLayer(self.inputs, name=name+'/input')
-            network_encode = DynamicRNN(
-                net_encode_in, cell_fn=cell_fn, cell_init_args=self.cell_init_args, n_hidden=n_hidden,
-                initializer=initializer, initial_state=initial_state_encode, dropout=dropout, n_layer=n_layer,
-                sequence_length=encode_sequence_length, return_last=False, return_seq_2d=True, name='encode'
-            )
-            # vs.reuse_variables()
-            # tl.layers.set_name_reuse(True)
-            network_decode = DynamicRNN(
-                net_decode_in, cell_fn=cell_fn, cell_init_args=self.cell_init_args, n_hidden=n_hidden,
-                initializer=initializer,
-                initial_state=(network_encode.final_state if initial_state_decode is None else initial_state_decode),
-                dropout=dropout, n_layer=n_layer, sequence_length=decode_sequence_length, return_last=False,
-                return_seq_2d=return_seq_2d, name='decode'
-            )
-            self.outputs = network_decode.outputs
-
-            # rnn_variables = tf.get_collection(TF_GRAPHKEYS_VARIABLES, scope=vs.name)
-
-        # Initial state
-        self.initial_state_encode = network_encode.initial_state
-        self.initial_state_decode = network_decode.initial_state
-
-        # Final state
-        self.final_state_encode = network_encode.final_state
-        self.final_state_decode = network_decode.final_state
-
-        # self.sequence_length = sequence_length
-        self._add_layers(network_encode.all_layers)
-        self._add_params(network_encode.all_params)
-        self._add_dropout_layers(network_encode.all_drop)
-
-        self._add_layers(network_decode.all_layers)
-        self._add_params(network_decode.all_params)
-        self._add_dropout_layers(network_decode.all_drop)
-
-        self._add_layers(self.outputs)
