@@ -4,6 +4,9 @@
 # """A collections of pre-defined well known models."""
 
 from .core import *
-from .squeezenetv1 import SqueezeNetV1
 from .mobilenetv1 import MobileNetV1
+from .resnet import ResNet50
+from .seq2seq import Seq2seq
+from .seq2seq_with_attention import Seq2seqLuongAttention
+from .squeezenetv1 import SqueezeNetV1
 from .vgg import *
