@@ -210,7 +210,8 @@ class Model(object):
             # check type of inputs and outputs
             check_order = ['inputs', 'outputs']
             for co, check_argu in enumerate([inputs, outputs]):
-                if isinstance(check_argu, tf_ops._TensorLike) or tf_ops.is_dense_tensor_like(check_argu):
+                if isinstance(check_argu,
+                              (tf.Tensor, tf.SparseTensor, tf.Variable)) or tf_ops.is_dense_tensor_like(check_argu):
                     pass
                 elif isinstance(check_argu, list):
                     if len(check_argu) == 0:
@@ -219,8 +220,9 @@ class Model(object):
                             "It should be either Tensor or a list of Tensor."
                         )
                     for idx in range(len(check_argu)):
-                        if not isinstance(check_argu[idx], tf_ops._TensorLike) or not tf_ops.is_dense_tensor_like(
-                                check_argu[idx]):
+                        if not isinstance(check_argu[idx],
+                                          (tf.Tensor, tf.SparseTensor, tf.Variable)) or not tf_ops.is_dense_tensor_like(
+                                              check_argu[idx]):
                             raise TypeError(
                                 "The argument `%s` should be either Tensor or a list of Tensor " % (check_order[co]) +
                                 "but the %s[%d] is detected as %s" % (check_order[co], idx, type(check_argu[idx]))
@@ -359,7 +361,12 @@ class Model(object):
             attr_list.remove("all_weights")
             attr_list.remove("trainable_weights")
             attr_list.remove("nontrainable_weights")
+            attr_list.remove("_all_weights")
+            attr_list.remove("_trainable_weights")
+            attr_list.remove("_nontrainable_weights")
             attr_list.remove("all_layers")
+            attr_list.remove("_all_layers")
+            attr_list.remove("n_weights")
             for idx, attr in enumerate(attr_list):
                 try:
                     if isinstance(getattr(self, attr), Layer):
@@ -402,7 +409,7 @@ class Model(object):
                 if layer.trainable_weights is not None:
                     self._trainable_weights.extend(layer.trainable_weights)
 
-        return self._trainable_weights
+        return self._trainable_weights.copy()
 
     @property
     def nontrainable_weights(self):
@@ -416,7 +423,7 @@ class Model(object):
                 if layer.nontrainable_weights is not None:
                     self._nontrainable_weights.extend(layer.nontrainable_weights)
 
-        return self._nontrainable_weights
+        return self._nontrainable_weights.copy()
 
     @property
     def all_weights(self):
@@ -430,7 +437,25 @@ class Model(object):
                 if layer.all_weights is not None:
                     self._all_weights.extend(layer.all_weights)
 
-        return self._all_weights
+        return self._all_weights.copy()
+
+    @property
+    def n_weights(self):
+        """Return the number of weights (parameters) in this network."""
+        n_weights = 0
+        for i, w in enumerate(self.all_weights):
+            n = 1
+            # for s in p.eval().shape:
+            for s in w.get_shape():
+                try:
+                    s = int(s)
+                except:
+                    s = 1
+                if s:
+                    n = n * s
+            n_weights = n_weights + n
+        # print("num of weights (parameters) %d" % n_weights)
+        return n_weights
 
     @property
     def config(self):
@@ -443,13 +468,14 @@ class Model(object):
                 _config.update({"name": None})
             else:
                 _config.update({"name": self.name})
-            # versionInfo = {
-            #     "tensorlayer_version": tl.__version__,
-            #     "backend": "tensorflow",
-            #     "backend_version": tf.__version__,
-            #     "training_device": "gpu",
-            # }
-            # _config.update(versionInfo)
+            version_info = {
+                "tensorlayer_version": tl.__version__,
+                "backend": "tensorflow",
+                "backend_version": tf.__version__,
+                "training_device": "gpu",
+                "save_date": None,
+            }
+            _config["version_info"] = version_info
             # if self.outputs is None:
             #     raise RuntimeError(
             #         "Dynamic mode does not support config yet."
@@ -591,6 +617,15 @@ class Model(object):
             layer._fix_nodes_for_layers()
         self._nodes_fixed = True
 
+    def __setattr__(self, key, value):
+        if isinstance(value, Layer):
+            if value._built is False:
+                raise AttributeError(
+                    "The registered layer `{}` should be built in advance. "
+                    "Do you forget to pass the keyword argument 'in_channels'? ".format(value.name)
+                )
+        super().__setattr__(key, value)
+
     def __repr__(self):
         # tmpstr = self.__class__.__name__ + '(\n'
         tmpstr = self.name + '(\n'
@@ -669,6 +704,8 @@ class Model(object):
 
         visited_node_names = set()
         for out_node in output_nodes:
+            if out_node.visited:
+                continue
             queue_node.put(out_node)
 
             while not queue_node.empty():
